unit calculator;

interface

uses
  Classes, Types, VCL.TMSFNCGraphics, VCL.TMSFNCCustomControl, VCL.TMSFNCGraphicsTypes, VCL.TMSFNCTypes, SysUtils;


type
  TCalculator = class(TTMSFNCCustomControl)
  private
    FMDown: integer;
    FValueStr: string;
    FValue: double;
    FLastValue: double;
    FLastOp: string;
  protected
    procedure HandleFontChanged(Sender: TObject); override;
    function XYToKey(X,Y: single): integer;
    procedure ProcessKey(AKey: integer);

    procedure HandleMouseDown({%H-}Button: TTMSFNCMouseButton; {%H-}Shift: TShiftState; {%H-}X, {%H-}Y: Single); override;
    procedure HandleMouseUp({%H-}Button: TTMSFNCMouseButton; {%H-}Shift: TShiftState; {%H-}X, {%H-}Y: Single); override;

    procedure Draw({%H-}AGraphics: TTMSFNCGraphics; {%H-}ARect: TRectF); override;

  public
    constructor Create(AOwner: TComponent); override;
  end;


implementation

{ TCalculator }

constructor TCalculator.Create(AOwner: TComponent);
begin
  inherited;
  Width := 300;
  Height := 375;
  FValue := 0;
  FLastValue := 0;
  FMDown := -1;
end;

procedure TCalculator.Draw(AGraphics: TTMSFNCGraphics; ARect: TRectF);
type
  row = array[0..3] of string;
var
  i,j: integer;
  dx,dy: single;
  keys: array[0..3] of row = (['7','8','9','+'],['4','5','6','-'],['1','2','3',''],['C','0','=','x']);
  r: TRectF;

begin
  inherited;
  dx := (Width -1) / 4;
  dy := (Height - 1) / 5;

  r := RectF(0,0,Width,dy);

  AGraphics.Font.Assign(Self.Font);

  AGraphics.DrawText(r, Format('%0g',[FValue]), false, gtaTrailing, gtaCenter);

  for i := 0 to 3 do
    for j := 1 to 4 do
      begin
        if FMDown = (i + (j-1) * 4) then
          AGraphics.Fill.Color := gcGray
        else
          AGraphics.Fill.Color := gcLightGray;

        AGraphics.DrawRectangle(i * dx, j * dy, (i+1) *dx, (j+1)*dy);
        r := RectF(i * dx, j * dy, (i+1) *dx, (j+1)*dy);
        AGraphics.DrawText(r, keys[j-1,i], false, gtaCenter, gtaCenter);
      end;

end;

procedure TCalculator.HandleFontChanged(Sender: TObject);
begin
  inherited;
  Invalidate;
end;

procedure TCalculator.HandleMouseDown(Button: TTMSFNCMouseButton;
  Shift: TShiftState; X, Y: Single);
begin
  inherited;
  FMDown := XYToKey(X,Y);
  Invalidate;
end;

procedure TCalculator.HandleMouseUp(Button: TTMSFNCMouseButton;
  Shift: TShiftState; X, Y: Single);
var
  AKey: integer;
begin
  inherited;

  AKey := XYToKey(X,Y);

  if AKey = FMDown then
  begin
    FMDown := -1;
    ProcessKey(AKey);
  end;
end;

procedure TCalculator.ProcessKey(AKey: integer);
var
  err: integer;
begin
  case AKey of
  0: FValueStr := FValueStr + '7';
  1: FValueStr := FValueStr + '8';
  2: FValueStr := FValueStr + '9';

  4: FValueStr := FValueStr + '4';
  5: FValueStr := FValueStr + '5';
  6: FValueStr := FValueStr + '6';

  8: FValueStr := FValueStr + '1';
  9: FValueStr := FValueStr + '2';
  10: FValueStr := FValueStr + '3';

  13: FValueStr := FValueStr + '0';

  3: FLastOp := '+';
  7: FLastOp := '-';
  11: FLastOp := '';
  15: FLastOp := 'x';

  12: FValueStr := '0';

  14:
    begin
      if FLastOp = '+' then
        FValue := FValue + FLastValue;

      if FLastOp = '-' then
        FValue := FLastValue - FValue;

      if FLastOp = '' then
        FValue := FLastValue / FValue;

      if FLastOp = 'x' then
        FValue := FLastValue * FValue;

      FValueStr := '';
    end;
  end;

  if AKey in [3,7,11,15] then
  begin
    FLastValue := FValue;
    FValueStr := '0';
  end;

  if AKey <> 14 then
    Val(FValueStr, FValue, err);

  Invalidate;
end;

function TCalculator.XYToKey(X, Y: single): integer;
var
  dx,dy: double;
  bx,by: integer;
begin
  dx := (Width -1) / 4;
  dy := (Height - 1) / 5;

  bx := Trunc(X/dx);
  by := Trunc(Y/dy);

  Result := bx + (by - 1) * 4;
end;

end.
